@@ -3,6 +3,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <search.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -23,6 +24,23 @@ static const char *data_member_base = "data.tar.";
  * Main udpkg implementation routines
  */
 
+static char *xasprintf(const char *format, ...)
+{
+	va_list args;
+	char *result;
+
+	va_start(args, format);
+	if (vasprintf(&result, format, args) < 0) {
+		if (errno == ENOMEM) {
+			fputs("Out of memory!\n", stderr);
+			abort();
+		}
+		return NULL;
+	}
+
+	return result;
+}
+
 static int is_file(const char *fn)
 {
 	struct stat statbuf;
@@ -213,7 +231,11 @@ static int dpkg_dounpack(struct package_t *pkg)
 		return 1;
 
 	cwd = getcwd(0, 0);
-	chdir("/");
+	if (chdir("/") != 0)
+	{
+		FPRINTF(stderr, "chdir /: %s\n", strerror(errno));
+		return 1;
+	}
 
 	snprintf(buf, sizeof(buf), "ar -p %s %s%s|%s -c|tar -x",
 		pkg->file, data_member_base,
@@ -330,7 +352,11 @@ static int dpkg_dounpack(struct package_t *pkg)
 	else
 		FPRINTF(stderr, "%s exited with status %d\n", buf, r);
 
-	chdir(cwd);
+	if (chdir(cwd) != 0)
+	{
+		FPRINTF(stderr, "chdir %s: %s\n", cwd, strerror(errno));
+		return 1;
+	}
 	return r;
 }
 
@@ -394,7 +420,11 @@ static int dpkg_unpackcontrol(struct package_t *pkg)
 	}
 	free(p);
 
-	chdir(cwd);
+	if (chdir(cwd) != 0)
+	{
+		FPRINTF(stderr, "chdir %s: %s\n", cwd, strerror(errno));
+		return 1;
+	}
 	free(cwd);
 	return 0;
 }
@@ -512,7 +542,7 @@ static int dpkg_fields(struct package_t *pkg)
 
 	reqarg(pkg);
 
-	asprintf(&command, "ar -p %s control.tar.gz|tar -xzOf - ./control", pkg->file);
+	command = xasprintf("ar -p %s control.tar.gz|tar -xzOf - ./control", pkg->file);
 	ret = system(command);
 	free(command);
 	return ret;
@@ -530,7 +560,7 @@ static int dpkg_contents(struct package_t *pkg)
 	if (compression_type == unknown_compression)
 		return 1;
 
-	asprintf(&command, "ar -p %s %s%s|%s|tar t",
+	command = xasprintf("ar -p %s %s%s|%s|tar t",
 		pkg->file,
 		data_member_base,
 		compression_extension(compression_type),
@@ -599,7 +629,7 @@ static int dpkg_remove(struct package_t *pkgs)
 #else
 	FPRINTF(stderr, "udpkg: No support for -r.\n");
 	return 1;
-	pkgs = 0; /* avoid -W warning */
+	(void)pkgs; /* avoid -W warning */
 #endif
 }
 
@@ -633,7 +663,7 @@ int main(int argc, char **argv)
 			if (**argv == '/')
 				p->file = *argv;
 			else
-				asprintf(&p->file, "%s/%s", cwd, *argv);
+				p->file = xasprintf("%s/%s", cwd, *argv);
 			p->package = strdup(*argv);
 			p->next = packages;
 			packages = p;
